#!/usr/bin/perl -w
###############################################################################
#
#       PROGRAM NAME- omwe-agree.pl
#	This program displays the statistics for the Sense tagged corpus
#	collected by the Open-Mind showing the #instances for each target 
#	word and along with the agreement and disagreement rates among the 
#	contributors who tagged these instances. 
#
###############################################################################
#                       ---------------------
#                       COPYRIGHT INFORMATION
#                       ---------------------

# Copyright (c) 2002,
# Amruta Purandare, University of Minnesota, Duluth
# pura0010@umn.edu
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to
#
# The Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
###############################################################################

#                       =============================
#                            THE CODE STARTS HERE
#                       =============================

##############################################################################
#                       ===============================
#                       COMMAND LINE OPTIONS AND USAGE
#                       ===============================

#show minimal usage
if($#ARGV<0)
{
        &minimal();
        exit;
}

use Getopt::Long;
GetOptions ("help","version","agree=i","disagree=i");

#command option for help
if(defined $opt_help)
{
        $opt_help=1;
        &showhelp();
        exit;
}
#version information
if(defined $opt_version)
{
        $opt_version=1;
        &showversion();
        exit;
}
#the agree option could be used to view only those words for which
#the agreement % is atleast as high as the specified threshold 
if(defined $opt_agree)
{
	$agree_lim=$opt_agree;
}

#the disagree option could be used to view only those words for which
#the dis-agreement % is atleast as high as the specified threshold
if(defined $opt_disagree)
{
        $disagree_lim=$opt_disagree;
}


##############################################################################

#                       ================================
#                          INITIALIZATION AND INPUT
#                       ================================

if(!defined $ARGV[0])
{
	print STDERR "ERROR:
        Please specify the name of the file that maps the instance-ids to the
	sense tags assigned by the contributors...\n";
        exit;
}

#accept the name of the file that contains the ids to sense tag mapping
$id2tag=$ARGV[0];

#check if the files exist and open if they do
if(!(-e $id2tag))
{
	print STDERR "ERROR:
The file mapping the instance id to the sense tags $id2tag doesn't exist...\n";
        exit;
}

open(ID2TAG,$id2tag) || die "Error in opening $id2tag file.\n";
# this file lists all the instance ids which don't have any tag assigned
open(OP_FILE,">output/notag2.txt") || die "Error in opening file notag.txt.\n";
##############################################################################

#			===========================================
#			 Actual Logic for finding the Distribution
#			===========================================

# reading the id2tag file and creating 2 tables 
# 1: showing the instances per target word - word2inst
# 2: listing all the tags assigned by different contributors for each instance
#	instance2tag
while($line=<ID2TAG>)
{
	# space trimming operations
	chomp $line;
        $line=~s/^\s+//g;
        $line=~s/\s$//g;
        #$line=~s/\s+/ /g;
	@entry=split(/\s+/,$line);
	#blank lines in file
        if($#entry<0)
        {
                next;
        }
	# case when more than one tags is assigned by the user for the same
        # sense which is an invalid case according to the open-mind README
        # we dont stop processing neither we skip this line but just show a
        # warning message
        if($#entry>1)
        {
                print STDERR "Warning:
        The entry $line has more than one tags by the same user for the same
        instance...\n";
        }
	# get the instance id part
        if(defined $entry[0])
        {
                $current_id=$entry[0];
        }
        else
        {
                next;
        }
	# target word in its base form is found in the beginning of the
        # instance id which is . separated
        ($part1,$part2,@rest)=split(/\./,$current_id);
	undef @rest;
	if((defined $part1) && (defined $part2))
	{
		 $word="$part1."."$part2";
	}
	else
	{
		print STDERR "ERROR:
	The instance id format at <$line> is invalid.\n";
		exit;
	}
	$word2inst{$word}{$current_id}++;
	foreach $sense_tag (1..$#entry)		
	{
		$instance2tag{$current_id}{$entry[$sense_tag]}++;	
	}
}

###	done with the file reading 		###
###	now finding the distribution !!! 	###

printf "%10s%15s%8s%10s%7s%10s%8s%12s","WORD","#INSTANCES","ONE-TAG", "MULTI-TAG","AGREE","DISAGREE","\%AGREE","\%DISAGREE\n";
foreach $word (sort keys %word2inst)
{
	$disagree=0;
	$once=0;
	$rest=0;
	# get all the instances for this target word
	@instances=keys %{$word2inst{$word}};
	# add to the total
	$total=$#instances+1;
	# consider each instance for all the tags assigned by different 
	# contributors
	foreach $instance (@instances)
	{
		$total_tags=0;
		# get the distinct tags
		@tags_assigned=keys %{$instance2tag{$instance}};	
		foreach (@tags_assigned)
		{
			$total_tags+=$instance2tag{$instance}{$_};
		}
		#print "<total tags=$total_tags\tinstance=$instance>";
		# this finds how many instances have particular number of tags
		$histo{$total_tags}++;
		# no tag  
		if($#tags_assigned==-1)
		{
			print OP_FILE "$instance\n";
		}
		# single tag assigned
		if($#tags_assigned==0 && $instance2tag{$instance}{$tags_assigned[0]}==1)
		{
			$once++;
		}
		# if more than one distinct tags for the same instance 
		# it is disagreement
		if($#tags_assigned>0)
		{
			$disagree++;
		}
	}
	# find dofferent values and their percentages
	$rest=$total-$once;
	$agree=$rest-$disagree;
	if($rest!=0)
	{
		$per_dis=$disagree/$rest*100;
		$per_agree=100-$per_dis;
	}
	else
	{
		$per_dis="undef";
		$per_agree="undef";
	}
	# if the options for filtering the words are selected use them
	# to show what is of interest
	if((defined $agree_lim && $per_agree>=$agree_lim)||(defined $disagree_lim && $per_dis>=$disagree_lim) || (!defined $agree_lim && !defined $disagree_lim))
	{
		printf "%10s\t",$word;
		printf "%5d\t",$total;
		printf "%5d\t%5d\t%7d\t%10d\t",$once,$rest,$agree,$disagree;
		if($per_dis ne "undef")
		{
			printf "%2.2f\t%2.2f\n", $per_agree,$per_dis;
		}
		else
		{
			printf "%5s\t%5s\n",$per_agree,$per_dis;
		}
		$global_total+=$total;
		$global_agree+=$agree;
	        $global_dis+=$disagree;
        	$global_once+=$once;
	        $global_rest+=$rest;
	}
} 

# final totals at the footer of the table1 
if($global_rest!=0)
{
	$global_per_agree=$global_agree/$global_rest*100;
	$global_per_dis=$global_dis/$global_rest*100;
}
else
{
	$global_per_agree="undef";
	$global_per_dis="undef";
}
printf "%10s\t%5d\t%5d\t%5d\t%7d\t%10d\t","Total",$global_total,$global_once,$global_rest,$global_agree,$global_dis;
if($global_per_agree ne "undef")
{
	printf "%2.2f\t%2.2f\n", $global_per_agree,$global_per_dis;
}
else
{
	printf "%5d\t%5d\n",$global_per_agree,$global_per_dis;
}

# showing the histogram of how many instances are assigned particular number 
# of tags
print "TABLE 2 INSTANCES\t#TAGS ASSIGNED\n";
@assigned_tags=keys %histo;
@sorted_tags=sort {$a <=> $b} @assigned_tags;
foreach $count_tag (@sorted_tags)
{
	print "\t$histo{$count_tag}\t\t$count_tag\n";
}
##############################################################################

#                       ==========================
#                           SUBROUTINE SECTION 
#                       ==========================

#-----------------------------------------------------------------------------
#show minimal usage message
sub minimal()
{
        print "Usage: omwe-agree.pl [OPTIONS] TAG_FILE\n";
        print "TYPE omwe-agree.pl --help for help\n";
}
#-----------------------------------------------------------------------------#show help
sub showhelp()
{
        print "This program displays 2 tables 
TABLE 1 shows -
The statistics for the Sense tagged Corpora collected by Open-Mind Word 
Experts showing rates of dis/agreement among the contributors who tagged same 
instance of a target word. 
2) The histogram showing number of instances having particular number of sense
tags assinged.";
        print "\n
Usage: omwe-agree.pl [OPTIONS] TAG_FILE \n";
        print "
TAG_FILE
        Each line of the TAG_FILE should show a tag assigned to an instance
        in the following format
        I       TAG
        Where I is an instance id and TAG shows a sense tag assigned to I
        by a contributor.
	
	TAG_FILE should follow the format specified by the Open Mind README. 
	omwe-agree.pl will terminate with error if the instance-ids found in 
	this file do not follow the standard format.\n";
	print "
OPTIONS:
--agree A
	Set value of A to any numeric value in [0-100] to see only those words
	which have \%agreement greater than or equal to threshold A.
--disagree D
	Set value of D to any numeric value in [0-100] to see only those words
        which have \%disagreement greater than or equal to threshold D.";
        print "
--help
        Get the help message.";
        print "
--version
        Get the version information.\n";
}

#------------------------------------------------------------------------------
#version information
sub showversion()
{
        print "omwe-agree.pl      -       Version 0.01";
	print "
This program shows the rate of agreement among different contributors who tag
the Open-Mind Data.\n";
        print "Copyright (C) 2002, Amruta Purandare and Ted Pedersen.\n";
        print "Date of Last Update:     01/06/2003\n";

}

